import dataclasses
import typing
from functools import partial

from django import http
from django.contrib import admin
from django.contrib.flatpages.admin import FlatPageAdmin as OriginalFlatPageAdmin
from django.contrib.flatpages.models import FlatPage as OriginalFlatPage
from django.db.models import Model
from django.urls import path, reverse
from django.utils.html import format_html
from django.utils.safestring import SafeString, mark_safe
from django.utils.translation import gettext_lazy
from django.views import View

from .. import models
from . import forms, views


class UnBoundRowActionException(Exception):
    pass


@dataclasses.dataclass
class RowAction:
    slug: str
    text: str
    url_name: str
    instance: Model | None = None

    def bind(self, instance: Model) -> typing.Self:
        return dataclasses.replace(self, instance=instance)

    @property
    def is_bound(self) -> bool:
        return self.instance is not None

    @property
    def url(self):
        if not self.is_bound:
            raise UnBoundRowActionException("RowAction must be bound")
        return reverse(self.url_name, args=[self.instance.pk])

    def as_html(self) -> SafeString:
        return format_html(
            '<li class="row-action"><a class="button fpe-action fpe-action-{slug}" href="{url}">{text}</a></li>',
            slug=self.slug,
            url=self.url,
            text=self.text,
        )


class ActionButtonColumnMixin:
    """
    Automatically add a column with action buttons
    """

    row_action_list: list[RowAction] = []

    def get_list_display(self, request: http.HttpRequest) -> list[str]:
        # Add `action_buttons` to `list_display` if it's not in there yet
        list_display = super().get_list_display(request)
        if "row_actions_column" not in list_display:
            list_display = [*list_display, "row_actions_column"]
        return list_display

    def get_row_action_list(self, obj: Model) -> list[RowAction]:
        actions = []
        for action in self.row_action_list:
            actions.append(action.bind(obj))
        return actions

    @admin.display(description=gettext_lazy("Actions"))
    def row_actions_column(self, obj) -> SafeString | None:
        actions = self.get_row_action_list(obj)
        if not actions:
            return None

        items = "\n".join(action.as_html() for action in actions)
        return mark_safe(f"<ul>\n{items}\n</ul>")


class AdminCBVMixin:
    def admin_cbv(self, view_class: View, **view_init_kwargs):
        if issubclass(view_class, views.ModelAdminMixin):
            view_init_kwargs["modeladmin"] = self

        return self.admin_site.admin_view(view_class.as_view(**view_init_kwargs))


admin.site.unregister(OriginalFlatPage)


@admin.register(models.FlatPage)
class FlatPageAdmin(ActionButtonColumnMixin, AdminCBVMixin, OriginalFlatPageAdmin):
    form = forms.FlatpageForm
    row_action_list = [
        RowAction(
            "addrevision",
            gettext_lazy("Add revision"),
            "admin:flatpages_extra_flatpage_addrevision",
        ),
    ]

    def get_urls(self):
        info = self.opts.app_label, self.opts.model_name

        return [
            path(
                "<int:pk>/add-revision/",
                self.admin_cbv(views.FlatPageCreateRevisionView),
                name="%s_%s_addrevision" % info,
            ),
            path(
                "_content_api/",
                self.admin_cbv(views.FlatPageContentAPIView),
                name="%s_%s_contentapi" % info,
            ),
            *super().get_urls(),
        ]


@admin.register(models.Revision)
class RevisionAdmin(ActionButtonColumnMixin, AdminCBVMixin, admin.ModelAdmin):
    form = forms.RevisionForm
    list_display = [
        "page",
        "description",
        "diffstats",
        "status",
    ]
    list_filter = ["status"]
    date_hierarchy = "created_on"
    row_action_list = [
        RowAction(
            "publish", gettext_lazy("Publish"), "admin:flatpages_extra_revision_publish"
        ),
        RowAction(
            "revert", gettext_lazy("Revert"), "admin:flatpages_extra_revision_revert"
        ),
        RowAction(
            "preview", gettext_lazy("Preview"), "admin:flatpages_extra_revision_preview"
        ),
        RowAction(
            "compare", gettext_lazy("Compare"), "admin:flatpages_extra_revision_diff"
        ),
    ]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "page":
            kwargs["widget"] = forms.PageForeignKeyWidget(db_field, self.admin_site)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_urls(self):
        info = self.opts.app_label, self.opts.model_name

        return [
            path(
                "<int:pk>/preview/",
                self.admin_cbv(views.RevisionPreviewRedirectView),
                name="%s_%s_preview" % info,
            ),
            path(
                "<int:pk>/diff/",
                self.admin_cbv(views.RevisionCompareView),
                name="%s_%s_diff" % info,
            ),
            path(
                "<int:pk>/publish/",
                self.admin_cbv(views.RevisionPublishView),
                name="%s_%s_publish" % info,
            ),
            path(
                "<int:pk>/revert/",
                self.admin_cbv(views.RevisionRevertView),
                name="%s_%s_revert" % info,
            ),
            *super().get_urls(),
        ]

    def get_row_action_list(self, obj):
        actions = []
        for action in super().get_row_action_list(obj):
            if (
                action.slug == "publish"
                and obj.status != models.Revision.Status.UNPUBLISHED
            ):
                continue
            if (
                action.slug == "revert"
                and obj.status != models.Revision.Status.PUBLISHED
            ):
                continue
            actions.append(action)
        return actions

    def get_form(self, request, obj=None, change=False, **kwargs):
        original_form_class = super().get_form(request, obj, change, **kwargs)

        form_class = partial(original_form_class, request=request)
        # Using a partial _almost_ works, but something somewhere later down the
        # line is introspecting the class's `base_fields`, so copy that too.
        form_class.base_fields = original_form_class.base_fields
        return form_class

    def get_fields(self, request, obj=None):
        fields = super().get_fields(request, obj)
        if obj is not None:
            fields.remove("page")
        return fields

    @admin.display(description="Lines changed")
    def diffstats(self, revision):
        return revision.diffstats_text



import functools

from django.contrib.admin.models import LogEntry
from django.contrib.auth.models import Group, Permission, User
from django.contrib.contenttypes.models import ContentType
from django.contrib.flatpages.models import FlatPage as OriginalFlatPage
from django.core.management.base import BaseCommand
from django.utils.translation import ngettext

from flatpages_extra.models import FlatPage, Revision

UserPermission = User.user_permissions.through
GroupPermission = Group.permissions.through


class Command(BaseCommand):
    help = "Import data from contrib.flatpages (log entries, permissions, and initial revisions)"

    @staticmethod
    @functools.cache
    def get_new_permission(old_permission):
        ct = ContentType.objects.get_for_model(FlatPage, for_concrete_model=False)
        return Permission.objects.get(
            content_type=ct,
            codename=old_permission.codename,
            name=old_permission.name,
        )

    def handle(self, **options):
        revisions = self.create_initial_revisions()
        if options["verbosity"] > 0:
            self.stdout.write(
                self.style.SUCCESS(
                    ngettext(
                        "Successfully imported {} revision",
                        "Successfully imported {} revisions",
                        len(revisions),
                    ).format(len(revisions))
                )
            )

        logentries = self.copy_logentries()
        if options["verbosity"] > 0:
            self.stdout.write(
                self.style.SUCCESS(
                    ngettext(
                        "Successfully copied {} log entry",
                        "Successfully copied {} log entries",
                        len(logentries),
                    ).format(len(logentries))
                )
            )

        custom_permissions = self.copy_custom_permissions()
        if options["verbosity"] > 0:
            self.stdout.write(
                self.style.SUCCESS(
                    ngettext(
                        "Successfully copied {} custom permission",
                        "Successfully copied {} custom permissions",
                        len(custom_permissions),
                    ).format(len(custom_permissions))
                )
            )

        user_permissions = self.copy_user_permissions()
        if options["verbosity"] > 0:
            self.stdout.write(
                self.style.SUCCESS(
                    ngettext(
                        "Successfully copied {} user permission",
                        "Successfully copied {} user permissions",
                        len(user_permissions),
                    ).format(len(user_permissions))
                )
            )

        group_permissions = self.copy_group_permissions()
        if options["verbosity"] > 0:
            self.stdout.write(
                self.style.SUCCESS(
                    ngettext(
                        "Successfully copied {} group permission",
                        "Successfully copied {} group permissions",
                        len(group_permissions),
                    ).format(len(group_permissions))
                )
            )

    def create_initial_revisions(self):
        revisions = [
            Revision(
                page=page,
                content=page.content,
                description="Initial flatpages-extra import.",
                status=Revision.Status.PUBLISHED,
            )
            for page in FlatPage.objects.all()
        ]
        return Revision.objects.bulk_create(revisions)

    def copy_logentries(self):
        ct_original = ContentType.objects.get_for_model(OriginalFlatPage)
        ct_new = ContentType.objects.get_for_model(FlatPage, for_concrete_model=False)

        entries = LogEntry.objects.filter(content_type=ct_original)
        for e in entries:
            e.pk = None
            e._state.adding = True
            e.content_type = ct_new

        return LogEntry.objects.bulk_create(entries)

    def copy_custom_permissions(self):
        ct_original = ContentType.objects.get_for_model(OriginalFlatPage)
        ct_new = ContentType.objects.get_for_model(FlatPage, for_concrete_model=False)

        permissions = []
        for p in Permission.objects.filter(content_type=ct_original):
            _, created = Permission.objects.get_or_create(
                content_type=ct_new,
                name=p.name,
                codename=p.codename,
            )
            if created:
                permissions.append(p)

        return permissions

    def copy_user_permissions(self):
        ct_original = ContentType.objects.get_for_model(OriginalFlatPage)
        user_permissions = UserPermission.objects.filter(
            permission__content_type=ct_original
        ).select_related("permission")
        for p in user_permissions:
            p.pk = None
            p._state.adding = True
            p.permission = self.get_new_permission(p.permission)

        return UserPermission.objects.bulk_create(user_permissions)

    def copy_group_permissions(self):
        ct_original = ContentType.objects.get_for_model(OriginalFlatPage)
        group_permissions = GroupPermission.objects.filter(
            permission__content_type=ct_original
        ).select_related("permission")
        for p in group_permissions:
            p.pk = None
            p._state.adding = True
            p.permission = self.get_new_permission(p.permission)

        return GroupPermission.objects.bulk_create(group_permissions)



class RevisionPublishError(Exception):
    pass


class RevisionPublishStatusError(RevisionPublishError):
    pass


class RevisionRevertError(Exception):
    pass


class RevisionRevertStatusError(RevisionRevertError):
    pass


class RevisionRevertContentError(RevisionRevertError):
    pass



from django.contrib import messages
from django.contrib.admin.helpers import AdminForm
from django.forms import Form
from django.http import JsonResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy
from django.views import generic
from django.views.generic.detail import BaseDetailView

from .. import models
from . import forms


class BlankForm(Form):
    pass


class ModelAdminMixin:
    modeladmin = None
    admin_title = None
    admin_subtitle = None

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(self.modeladmin.admin_site.each_context(self.request))
        adminform = AdminForm(
            form=context.get("form", BlankForm()),
            fieldsets=[],
            prepopulated_fields={},
        )
        context.update(
            title=self.admin_title,
            subtitle=self.admin_subtitle,
            adminform=adminform,
            object_id=self.object.pk,
            original=self.object,
            is_popup=False,
            to_field=None,
            media=adminform.media,
            inline_admin_formsets=[],
            errors=None,
            preserved_filters=None,
        )
        # This will mutate the passed context, we can discard the return value:
        self.modeladmin.render_change_form(self.request, context, obj=self.object)
        return context


class FlatPageCreateRevisionView(generic.RedirectView):
    pattern_name = "admin:flatpages_extra_revision_add"

    def get_redirect_url(self, pk):
        url = super().get_redirect_url()
        return f"{url}?page={pk}"


class FlatPageContentAPIView(generic.View):
    def get(self, request):
        page = get_object_or_404(models.FlatPage, pk=request.GET.get("pk"))
        data = {"content": page.content}
        return JsonResponse(data)


class RevisionPreviewRedirectView(BaseDetailView):
    model = models.Revision

    def render_to_response(self, context):
        return redirect(context["object"])


class RevionsBaseActionView(ModelAdminMixin, generic.UpdateView):
    model = models.Revision
    success_message = None

    def get_success_message(self):
        return self.success_message

    def get_page_logentry_message(self):
        return self.object.description

    def form_valid(self, form):
        response = super().form_valid(form)
        self.modeladmin.log_change(
            request=self.request,
            obj=self.object.page,
            message=self.get_page_logentry_message(),
        )
        if success_message := self.get_success_message():
            messages.success(self.request, success_message)
        return response


class RevisionPublishView(RevionsBaseActionView):
    template_name = "admin/flatpages_extra/revision/publish.html"
    form_class = forms.RevisionPublishForm
    admin_title = "Publish revision"
    admin_subtitle = "Are you sure you want to publish this revision?"
    success_message = gettext_lazy(
        "The revision has been published. The new page should be live."
    )
    success_url = reverse_lazy("admin:flatpages_extra_revision_changelist")


class RevisionRevertView(RevionsBaseActionView):
    template_name = "admin/flatpages_extra/revision/revert.html"
    form_class = forms.RevisionRevertForm
    admin_title = "Revert revision"
    admin_subtitle = "Are you sure you want to revert this revision?"
    success_message = gettext_lazy(
        "The revision has been reverted. The old version should be live again."
    )
    success_url = reverse_lazy("admin:flatpages_extra_revision_changelist")

    def get_page_logentry_message(self):
        return f'Reverted "{self.object.description}"'


class RevisionCompareView(ModelAdminMixin, generic.DetailView):
    template_name = "admin/flatpages_extra/revision/compare.html"
    model = models.Revision
    admin_title = "Comparing revision with original"



from django import forms
from django.contrib.admin.widgets import AdminTextareaWidget, AutocompleteSelect
from django.contrib.flatpages.forms import FlatpageForm as OriginalFlatpageForm
from django.urls import reverse

from .. import models


class ContentWidget(AdminTextareaWidget):
    template_name = "admin/flatpages_extra/flatpage/widgets/content.html"

    def __init__(self, *args, page_pk=None, **kwargs):
        self.page_pk = page_pk
        super().__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["widget"]["page_pk"] = self.page_pk
        return context


class FlatpageForm(OriginalFlatpageForm):
    class Meta:
        model = models.FlatPage
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.instance._state.adding:
            self.fields["content"].widget = ContentWidget(page_pk=self.instance.pk)


class PageForeignKeyWidget(AutocompleteSelect):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.attrs.update(
            {
                "data-fpe-contentapi-url": reverse(
                    "admin:flatpages_extra_flatpage_contentapi"
                )
            }
        )

    @property
    def media(self):
        custom_js = [
            "admin/js/jquery.init.js",
            "admin/flatpages_extra/page-fk-widget.js",
        ]
        return super().media + forms.Media(js=custom_js)


class RevisionForm(forms.ModelForm):
    def __init__(self, *args, request, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)
        if pagepk := self.initial.get("page"):
            # For some reason the js triggers anyway, so this might not be necessary
            self.instance.page = models.FlatPage.objects.get(pk=pagepk)
            self.initial.update(
                content=self.instance.page.content,
            )

    def _save_m2m(self):
        super()._save_m2m()
        self.instance.authors.add(self.request.user)

    class Meta:
        widgets = {
            "description": forms.TextInput(attrs={"class": "vLargeTextField"}),
        }


class RevisionBaseActionForm(forms.ModelForm):
    method_name = None

    class Meta:
        model = models.Revision
        fields = []

    def save(self, commit=True):
        if not commit:
            raise NotImplementedError
        getattr(self.instance, self.method_name)(save=True)
        return self.instance


class RevisionPublishForm(RevisionBaseActionForm):
    method_name = "publish"


class RevisionRevertForm(RevisionBaseActionForm):
    method_name = "revert"



import difflib
import typing
from collections import abc
from dataclasses import dataclass

from django.contrib.flatpages.models import FlatPage as OriginalFlatPage
from django.db import models, transaction
from django.urls import reverse
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.safestring import SafeString, mark_safe
from django.utils.translation import gettext_lazy
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import DiffLexer

from . import exceptions

DiffType = abc.Iterable[str]


class FlatPage(OriginalFlatPage):
    class Meta:
        proxy = True


def get_random_share_key() -> str:
    return get_random_string(10)


def get_content_field(**kwargs) -> models.Field:
    """
    Return a copy of the original flatpages' `content` field, with the given
    kwargs overridden.
    """
    original_field = OriginalFlatPage._meta.get_field("content")
    _, _, construct_args, construct_kwargs = original_field.deconstruct()
    construct_kwargs.update(kwargs)
    return original_field.__class__(*construct_args, **construct_kwargs)


@dataclass
class DiffStat:
    plus: int
    minus: int

    @classmethod
    def from_unified_diff(cls, diff: DiffType) -> typing.Self:
        # the first two lines are `---` and `+++` and I don't want them to be counted
        plus, minus = -1, -1
        for diffline in diff:
            if diffline.startswith("+"):
                plus += 1
            elif diffline.startswith("-"):
                minus += 1

        return cls(plus, minus)

    def __str__(self):
        return f"+{self.plus} / -{self.minus}"


class Revision(models.Model):
    class Status(models.IntegerChoices):
        UNPUBLISHED = 1
        PUBLISHED = 2
        REVERTED = 3

    # Local alias for exceptions for quicker access
    PublishError = exceptions.RevisionPublishError
    PublishStatusError = exceptions.RevisionPublishStatusError
    RevertError = exceptions.RevisionRevertError
    RevertStatusError = exceptions.RevisionRevertStatusError
    RevertContentError = exceptions.RevisionRevertContentError

    page = models.ForeignKey(FlatPage, on_delete=models.CASCADE, related_name="changes")
    content = get_content_field()
    previous_content = get_content_field(editable=False, blank=True)
    description = models.TextField(
        verbose_name=gettext_lazy("Description of the change"),
        help_text=gettext_lazy(
            "Describe the changes you've made. Future you will be thankful."
        ),
    )
    status = models.IntegerField(
        choices=Status, default=Status.UNPUBLISHED, editable=False
    )
    authors = models.ManyToManyField("auth.User", blank=True, editable=False)
    created_on = models.DateTimeField(auto_now_add=True, editable=False)
    updated_on = models.DateTimeField(auto_now=True, editable=False)
    published_on = models.DateTimeField(blank=True, null=True, editable=False)
    reverted_on = models.DateTimeField(blank=True, null=True, editable=False)
    share_key = models.CharField(
        max_length=10, default=get_random_share_key, editable=False
    )

    class Meta:
        ordering = ("-updated_on",)

    def __str__(self):
        return f"Revision #{self.pk}: {self.diffstats_text}"

    def get_absolute_url(self) -> str:
        from .views import preview  # avoid circular import

        return reverse(preview, kwargs={"pk": self.pk, "key": self.share_key})

    def raise_for_publish(self) -> None:
        if self.status != self.Status.UNPUBLISHED:
            raise self.PublishStatusError("Can only apply unpublished revisions")

    @property
    def can_publish(self) -> bool:
        try:
            self.raise_for_publish()
        except self.PublishError:
            return False
        else:
            return True

    def raise_for_revert(self) -> None:
        if self.status != self.Status.PUBLISHED:
            raise self.RevertStatusError("Can only revert published revisions")
        if self.page.content != self.content:
            raise self.RevertContentError(
                "Page has been modified since revision was published"
            )

    @property
    def can_revert(self) -> bool:
        try:
            self.raise_for_revert()
        except self.RevertError:
            return False
        else:
            return True

    def publish(self, save: bool = False, skip_check: bool = False) -> None:
        if not skip_check:
            self.raise_for_publish()

        self.previous_content = self.page.content
        self.page.content = self.content
        self.status = self.Status.PUBLISHED
        self.published_on = timezone.now()

        if save:
            with transaction.atomic():
                self.page.save(update_fields=["content"])
                self.save(update_fields=["status", "previous_content", "published_on"])

    def revert(self, save: bool = False, skip_check: bool = False) -> None:
        if not skip_check:
            self.raise_for_revert()

        self.page.content = self.previous_content
        self.status = self.Status.REVERTED
        self.reverted_on = timezone.now()
        if save:
            with transaction.atomic():
                self.page.save(update_fields=["content"])
                self.save(update_fields=["status", "reverted_on"])

    @property
    def original_content(self) -> str:
        if self.status == self.Status.UNPUBLISHED:
            return self.page.content
        else:
            return self.previous_content

    @property
    def unified_diff(self) -> DiffType:
        return difflib.unified_diff(
            self.original_content.splitlines(),
            self.content.splitlines(),
            lineterm="",
        )

    @property
    def highlighted_diff(self) -> SafeString:
        return mark_safe(
            highlight(
                "\n".join(self.unified_diff),
                DiffLexer(),
                HtmlFormatter(),
            )
        )

    @property
    def diffstats_text(self) -> str:
        return str(DiffStat.from_unified_diff(self.unified_diff))



# Generated by Django 5.2.4 on 2025-07-14 22:14

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import flatpages_extra.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("flatpages", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="FlatPage",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("flatpages.flatpage",),
        ),
        migrations.CreateModel(
            name="Revision",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", models.TextField(blank=True, verbose_name="content")),
                (
                    "previous_content",
                    models.TextField(
                        blank=True, editable=False, verbose_name="content"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        help_text="Describe the changes you've made. Future you will be thankful.",
                        verbose_name="Description of the change",
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[(1, "Unpublished"), (2, "Published"), (3, "Reverted")],
                        default=1,
                        editable=False,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
                (
                    "published_on",
                    models.DateTimeField(blank=True, editable=False, null=True),
                ),
                (
                    "reverted_on",
                    models.DateTimeField(blank=True, editable=False, null=True),
                ),
                (
                    "share_key",
                    models.CharField(
                        default=flatpages_extra.models.get_random_share_key,
                        editable=False,
                        max_length=10,
                    ),
                ),
                ("authors", models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
                (
                    "page",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="changes",
                        to="flatpages_extra.flatpage",
                    ),
                ),
            ],
            options={
                "ordering": ("-updated_on",),
            },
        ),
    ]



from django import http
from django.contrib import messages
from django.contrib.flatpages.views import render_flatpage
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext as _
from django.views.decorators.cache import never_cache

from .models import Revision


@never_cache
def preview(request: http.HttpRequest, pk: int, key: str) -> http.HttpResponse:
    queryset = Revision.objects.select_related("page")
    revision = get_object_or_404(queryset, pk=pk, share_key=key)
    revision.publish(save=False, skip_check=True)
    warning_message = _("This is a preview page, do not share.")
    if request.user.has_perm("flatpages_extra.change_revision"):
        warning_message += format_html(
            ' <a href="{}">{}</a>',
            reverse("admin:flatpages_extra_revision_change", args=[revision.pk]),
            _("Edit it in the admin."),
        )
    messages.warning(request, warning_message)
    return render_flatpage(request, revision.page)



from django.apps import AppConfig


class FlatPagesPlusConfig(AppConfig):
    name = "flatpages_extra"
    verbose_name = "Flat Pages 💅"

    def ready(self):
        from . import checks  # noqa: F401 (imported for registration)



from django.apps import apps
from django.conf import settings
from django.core.checks import Error, register
from django.urls import NoReverseMatch

from . import models

E001_URLS_NOT_INSTALLED_ERROR = Error(
    "flatpages_extra URLs are not installed",
    hint=(
        'Add `path("flatpagesextra/", include("flatpages_extra.urls"))` '
        f"to your root urlconf ({settings.ROOT_URLCONF}).",
    ),
    id="flatpages_extra.E001",
)


E002_CONTRIB_FLATPAGES_NOT_INSTALLED_ERROR = Error(
    "contrib.flatpages is not installed",
    hint="Add 'contrib.flatpages' to settings.INSTALLED_APPS.",
    id="flatpages_extra.E002",
)


def check_urls_are_installed():
    revision = models.Revision(pk=123, share_key="asdf")
    try:
        revision.get_absolute_url()
    except NoReverseMatch:
        yield E001_URLS_NOT_INSTALLED_ERROR


def check_contrib_flatpages_is_installed():
    if not apps.is_installed("django.contrib.flatpages"):
        yield E002_CONTRIB_FLATPAGES_NOT_INSTALLED_ERROR


@register
def check_all(app_configs, **kwargs):
    errors = []
    errors.extend(check_urls_are_installed())
    errors.extend(check_contrib_flatpages_is_installed())

    return errors
